@@ -88,18 +88,28 @@ pub struct PrecompileOutput {
 	pub logs: Vec<Log>,
 }
 
+/// Precompiles function signature. Expected input arguments are:
+///  * Address
+///  * Input
+///  * Context
+///  * State
+///  * Is static
+type PrecompileFn<S> = fn(H160, &[u8], Option<u64>, &Context, &mut S, bool) -> Option<Result<PrecompileOutput, ExitError>>;
+
 /// Stack-based executor.
 pub struct StackExecutor<'config, S> {
 	config: &'config Config,
-	precompile: fn(H160, &[u8], Option<u64>, &Context) -> Option<Result<PrecompileOutput, ExitError>>,
+	precompile: PrecompileFn<S>,
 	state: S,
 }
 
-fn no_precompile(
+fn no_precompile<S>(
 	_address: H160,
 	_input: &[u8],
 	_target_gas: Option<u64>,
 	_context: &Context,
+	_state: &mut S,
+	_is_static: bool,
 ) -> Option<Result<PrecompileOutput, ExitError>> {
 	None
 }
@@ -124,7 +134,7 @@ impl<'config, S: StackState<'config>> StackExecutor<'config, S> {
 	pub fn new_with_precompile(
 		state: S,
 		config: &'config Config,
-		precompile: fn(H160, &[u8], Option<u64>, &Context) -> Option<Result<PrecompileOutput, ExitError>>,
+		precompile: PrecompileFn<S>,
 	) -> Self {
 		Self {
 			config,
@@ -551,7 +561,7 @@ impl<'config, S: StackState<'config>> StackExecutor<'config, S> {
 			}
 		}
 
-		if let Some(ret) = (self.precompile)(code_address, &input, Some(gas_limit), &context) {
+		if let Some(ret) = (self.precompile)(code_address, &input, Some(gas_limit), &context, &mut self.state, is_static) {
 			match ret {
 				Ok(PrecompileOutput { exit_status , output, cost, logs }) => {
 					for Log { address, topics, data} in logs {
